open Misc

let _ =
  if !what = Compile then
    what := Liveness

let print_code =
  List.iter (fun i -> Ass.nformat Spim.namer i; print_newline())

let print_spim pfun p =
  let g {Spim.code=c ; Spim.frame=f} = pfun f c in
  List.iter g (p.Spim.procedures) ; g p.Spim.main ;
  p

let ( * ) f g x = f (g x)


let compile =  
  Spim.program * Linear_opt.program * Trans.program

let compile_spill = Spill.spill_all * compile


let main () = match !prog with
| "" -> ()
| name ->
    let lexbuf = Lexing.from_channel (open_in name)  in
    Location.set name lexbuf ;
    let r = Parser.main Lexer.token lexbuf in
    match !what with
    | Pretty ->  Print.program r
    | IPretty|IFlowGraph ->
        let _ =  Linear_opt.program (Trans.program r) in
        ()
    | Simul -> Simul.program (Linear_opt.program (Trans.program r))
    | Spim ->
        let r = compile r in
        let _ = print_spim
            (fun _ is ->
              List.iter (fun i -> Ass.nformat Spim.namer i; print_newline())
                is)
            r in
        ()
    | Liveness -> 
        let r = compile r in
        let prog_name =
          Filename.basename (Filename.chop_extension !Misc.prog) in
        let _ = print_spim
            (fun f c ->
              let fg = Liveness.flow c in
              Printf.printf "*** Graphe de flot ***\n" ;
              Liveness.print_flowcode stdout fg ;
              Printf.printf "*** Graphe des interfrences ***\n" ;
              let chan =
                open_out (prog_name ^ "-" ^
                          Gen.label_string (Frame.frame_name f)^".ig") in
              Printf.fprintf chan "graph {\n" ;
              Liveness.print_interference stdout
                chan (Gen.label_string (Frame.frame_name f))
                (Liveness.interference fg) ;
              Printf.fprintf chan "}\n" ;
              close_out chan)
            r in
        ()
    | _ ->
        prerr_endline "Pour la suite du cours !!!" ;
        exit 2

;;

try
  main ()
with
| Parsing.Parse_error ->
    Location.print_pos () ;
    prerr_endline "Syntax error" ;
    exit 2
| Lexer.Error ->
    Location.print_pos () ;
    prerr_endline "Lexing error" ;
    exit 2 
| e ->
    prerr_endline ("Exception baladeuse : "^Printexc.to_string e) ;
    exit 2
;;

